@@ -118,10 +118,9 @@ SPECIAL FLAGS: There are a few flags that have special meaning:
    --helpshort     prints a list of all key flags (see below).
    --helpxml       prints a list of all flags, in XML format.  DO NOT parse
                    the output of --help and --helpshort.  Instead, parse
-                   the output of --helpxml.  As we add new flags, we may
-                   add new XML elements.  Hence, make sure your parser
-                   does not crash when it encounters new XML elements.
-   --flagfile=foo  read flags from foo.
+                   the output of --helpxml.  For more info, see
+                   "OUTPUT FOR --helpxml" below.
+   --flagfile=foo  read flags from file foo.
    --undefok=f1,f2 ignore unrecognized option errors for f1,f2.
                    For boolean flags, you should use --undefok=boolflag, and
                    --boolflag and --noboolflag will be accepted.  Do not use
@@ -129,6 +128,47 @@ SPECIAL FLAGS: There are a few flags that have special meaning:
    --              as in getopt(), terminates flag-processing
 
 
+FLAGS VALIDATORS: If your program:
+  - requires flag X to be specified
+  - needs flag Y to match a regular expression
+  - or requires any more general constraint to be satisfied
+then validators are for you!
+
+Each validator represents a constraint over one flag, which is enforced
+starting from the initial parsing of the flags and until the program
+terminates.
+
+Also, lower_bound and upper_bound for numerical flags are enforced using flag
+validators.
+
+Howto:
+If you want to enforce a constraint over one flag, use
+
+flags.RegisterValidator(flag_name,
+                        checker,
+                        message='Flag validation failed',
+                        flag_values=FLAGS)
+
+After flag values are initially parsed, and after any change to the specified
+flag, method checker(flag_value) will be executed. If constraint is not
+satisfied, an IllegalFlagValue exception will be raised. See
+RegisterValidator's docstring for a detailed explanation on how to construct
+your own checker.
+
+
+EXAMPLE USAGE:
+
+FLAGS = flags.FLAGS
+
+flags.DEFINE_integer('my_version', 0, 'Version number.')
+flags.DEFINE_string('filename', None, 'Input file name')
+
+flags.RegisterValidator('my_version',
+                        lambda value: value % 2 == 0,
+                        message='--my_version must be divisible by 2')
+flags.MarkFlagAsRequired('filename')
+
+
 NOTE ON --flagfile:
 
 Flags may be loaded from text files in addition to being specified on
@@ -297,6 +337,53 @@ just --num_replicas) or by any of the modules it transitively imports
 (e.g., the modules libfoo, libbar).  E.g., it can access the value of
 FLAGS.bar_risky_hack, even if --bar_risky_hack is not declared as a key
 flag for myscript.
+
+
+OUTPUT FOR --helpxml:
+
+The --helpxml flag generates output with the following structure:
+
+<?xml version="1.0"?>
+<AllFlags>
+  <program>PROGRAM_BASENAME</program>
+  <usage>MAIN_MODULE_DOCSTRING</usage>
+  (<flag>
+    [<key>yes</key>]
+    <file>DECLARING_MODULE</file>
+    <name>FLAG_NAME</name>
+    <meaning>FLAG_HELP_MESSAGE</meaning>
+    <default>DEFAULT_FLAG_VALUE</default>
+    <current>CURRENT_FLAG_VALUE</current>
+    <type>FLAG_TYPE</type>
+    [OPTIONAL_ELEMENTS]
+  </flag>)*
+</AllFlags>
+
+Notes:
+
+1. The output is intentionally similar to the output generated by the
+C++ command-line flag library.  The few differences are due to the
+Python flags that do not have a C++ equivalent (at least not yet),
+e.g., DEFINE_list.
+
+2. New XML elements may be added in the future.
+
+3. DEFAULT_FLAG_VALUE is in serialized form, i.e., the string you can
+pass for this flag on the command-line.  E.g., for a flag defined
+using DEFINE_list, this field may be foo,bar, not ['foo', 'bar'].
+
+4. CURRENT_FLAG_VALUE is produced using str().  This means that the
+string 'false' will be represented in the same way as the boolean
+False.  Using repr() would have removed this ambiguity and simplified
+parsing, but would have broken the compatibility with the C++
+command-line flags.
+
+5. OPTIONAL_ELEMENTS describe elements relevant for certain kinds of
+flags: lower_bound, upper_bound (for flags that specify bounds),
+enum_value (for enum flags), list_separator (for flags that consist of
+a list of values, separated by a special token).
+
+6. We do not provide any example here: please use --helpxml instead.
 """
 
 import cgi
@@ -306,15 +393,17 @@ import re
 import string
 import sys
 
-# Are we running at least python 2.2?                                           
+import gflags_validators
+
+# Are we running at least python 2.2?
 try:
   if tuple(sys.version_info[:3]) < (2,2,0):
     raise NotImplementedError("requires python 2.2.0 or later")
-except AttributeError:   # a very old python, that lacks sys.version_info       
+except AttributeError:   # a very old python, that lacks sys.version_info
   raise NotImplementedError("requires python 2.2.0 or later")
 
-# If we're not running at least python 2.2.1, define True, False, and bool.     
-# Thanks, Guido, for the code.                                                  
+# If we're not running at least python 2.2.1, define True, False, and bool.
+# Thanks, Guido, for the code.
 try:
   True, False, bool
 except NameError:
@@ -339,12 +428,18 @@ def _GetCallingModule():
   # Walk down the stack to find the first globals dict that's not ours.
   for depth in range(1, sys.getrecursionlimit()):
     if not sys._getframe(depth).f_globals is globals():
-      module_name = __GetModuleName(sys._getframe(depth).f_globals)
+      globals_for_frame = sys._getframe(depth).f_globals
+      module_name = _GetModuleObjectAndName(globals_for_frame)[1]
       if module_name is not None:
         return module_name
   raise AssertionError("No module was found")
 
 
+def _GetThisModuleObjectAndName():
+  """Returns: (module object, module name) for this module."""
+  return _GetModuleObjectAndName(globals())
+
+
 # module exceptions:
 class FlagsError(Exception):
   """The base class for all flags errors."""
@@ -356,6 +451,17 @@ class DuplicateFlag(FlagsError):
   pass
 
 
+class DuplicateFlagCannotPropagateNoneToSwig(DuplicateFlag):
+  """Special case of DuplicateFlag -- SWIG flag value can't be set to None.
+
+  This can be raised when a duplicate flag is created. Even if allow_override is
+  True, we still abort if the new value is None, because it's currently
+  impossible to pass None default value back to SWIG. See FlagValues.SetDefault
+  for details.
+  """
+  pass
+
+
 # A DuplicateFlagError conveys more information than a
 # DuplicateFlag. Since there are external modules that create
 # DuplicateFlags, the interface to DuplicateFlag shouldn't change.
@@ -384,16 +490,20 @@ class UnrecognizedFlag(FlagsError):
   pass
 
 
-# An UnrecognizedFlagError conveys more information than an
-# UnrecognizedFlag. Since there are external modules that create
-# DuplicateFlags, the interface to DuplicateFlag shouldn't change.
+# An UnrecognizedFlagError conveys more information than an UnrecognizedFlag.
+# Since there are external modules that create DuplicateFlags, the interface to
+# DuplicateFlag shouldn't change.  The flagvalue will be assigned the full value
+# of the flag and its argument, if any, allowing handling of unrecognzed flags
+# in an exception handler.
+# If flagvalue is the empty string, then this exception is an due to a
+# reference to a flag that was not already defined.
 class UnrecognizedFlagError(UnrecognizedFlag):
-  def __init__(self, flagname):
+  def __init__(self, flagname, flagvalue=''):
     self.flagname = flagname
+    self.flagvalue = flagvalue
     UnrecognizedFlag.__init__(
         self, "Unknown command line flag '%s'" % flagname)
 
-
 # Global variable used by expvar
 _exported_flags = {}
 _help_width = 80  # width of help output
@@ -586,23 +696,29 @@ def DocToHelp(doc):
   return doc
 
 
-def __GetModuleName(globals_dict):
-  """Given a globals dict, returns the name of the module that defines it.
+def _GetModuleObjectAndName(globals_dict):
+  """Returns the module that defines a global environment, and its name.
 
   Args:
     globals_dict: A dictionary that should correspond to an environment
       providing the values of the globals.
 
   Returns:
-    A string (the name of the module) or None (if the module could not
-    be identified.
+    A pair consisting of (1) module object and (2) module name (a
+    string).  Returns (None, None) if the module could not be
+    identified.
   """
-  for name, module in sys.modules.iteritems():
+  # The use of .items() (instead of .iteritems()) is NOT a mistake: if
+  # a parallel thread imports a module while we iterate over
+  # .iteritems() (not nice, but possible), we get a RuntimeError ...
+  # Hence, we use the slightly slower but safer .items().
+  for name, module in sys.modules.items():
     if getattr(module, '__dict__', None) is globals_dict:
       if name == '__main__':
-        return sys.argv[0]
-      return name
-  return None
+        # Pick a more informative name for the main module.
+        name = sys.argv[0]
+      return (module, name)
+  return (None, None)
 
 
 def _GetMainModule():
@@ -611,7 +727,7 @@ def _GetMainModule():
     try:
       globals_of_main = sys._getframe(depth).f_globals
     except ValueError:
-      return __GetModuleName(globals_of_main)
+      return _GetModuleObjectAndName(globals_of_main)[1]
   raise AssertionError("No module was found")
 
 
@@ -775,6 +891,15 @@ class FlagValues:
       if flag_name == flag.name:
         self[flag_name] = flag
 
+  def RemoveFlagValues(self, flag_values):
+    """Remove flags that were previously appended from another FlagValues.
+
+    Args:
+      flag_values: registry containing flags to remove.
+    """
+    for flag_name in flag_values.FlagDict():
+      self.__delattr__(flag_name)
+
   def __setitem__(self, name, flag):
     """Registers a new flag variable."""
     fl = self.FlagDict()
@@ -814,8 +939,35 @@ class FlagValues:
     """Sets the 'value' attribute of the flag --name."""
     fl = self.FlagDict()
     fl[name].value = value
+    self._AssertValidators(fl[name].validators)
     return value
 
+  def _AssertAllValidators(self):
+    all_validators = set()
+    for flag in self.FlagDict().itervalues():
+      for validator in flag.validators:
+        all_validators.add(validator)
+    self._AssertValidators(all_validators)
+
+  def _AssertValidators(self, validators):
+    """Assert if all validators in the list are satisfied.
+
+    Asserts validators in the order they were created.
+    Args:
+      validators: Iterable(gflags_validators.Validator), validators to be
+        verified
+    Raises:
+      AttributeError: if validators work with a non-existing flag.
+      IllegalFlagValue: if validation fails for at least one validator
+    """
+    for validator in sorted(
+        validators, key=lambda validator: validator.insertion_index):
+      try:
+        validator.Verify(self)
+      except gflags_validators.Error, e:
+        message = validator.PrintFlagsWithValues(self)
+        raise IllegalFlagValue('%s: %s' % (message, str(e)))
+
   def _FlagIsRegistered(self, flag_obj):
     """Checks whether a Flag object is registered under some name.
 
@@ -898,6 +1050,7 @@ class FlagValues:
     if not fl.has_key(name):
       raise AttributeError(name)
     fl[name].SetDefault(value)
+    self._AssertValidators(fl[name].validators)
 
   def __contains__(self, name):
     """Returns True if name is a value (flag) in the dict."""
@@ -934,7 +1087,7 @@ class FlagValues:
     fl = self.FlagDict()
 
     # This pre parses the argv list for --flagfile=<> options.
-    argv = self.ReadFlagsFromFiles(argv)
+    argv = argv[:1] + self.ReadFlagsFromFiles(argv[1:], force_gnu=False)
 
     # Correct the argv to support the google style of passing boolean
     # parameters.  Boolean parameters may be passed by using --mybool,
@@ -993,13 +1146,12 @@ class FlagValues:
         if not e.opt or e.opt in fl:
           # Not an unrecognized option, reraise the exception as a FlagsError
           raise FlagsError(e)
-        # Handle an unrecognized option.
-        unrecognized_opts.append(e.opt)
         # Remove offender from args and try again
         for arg_index in range(len(args)):
           if ((args[arg_index] == '--' + e.opt) or
               (args[arg_index] == '-' + e.opt) or
-              args[arg_index].startswith('--' + e.opt + '=')):
+              (args[arg_index].startswith('--' + e.opt + '='))):
+            unrecognized_opts.append((e.opt, args[arg_index]))
             args = args[0:arg_index] + args[arg_index+1:]
             break
         else:
@@ -1037,21 +1189,24 @@ class FlagValues:
 
     # If there were unrecognized options, raise an exception unless
     # the options were named via --undefok.
-    for opt in unrecognized_opts:
+    for opt, value in unrecognized_opts:
       if opt not in undefok_flags:
-        raise UnrecognizedFlagError(opt)
+        raise UnrecognizedFlagError(opt, value)
 
     if unparsed_args:
       if self.__dict__['__use_gnu_getopt']:
         # if using gnu_getopt just return the program name + remainder of argv.
-        return argv[:1] + unparsed_args
+        ret_val = argv[:1] + unparsed_args
       else:
         # unparsed_args becomes the first non-flag detected by getopt to
         # the end of argv.  Because argv may have been modified above,
         # return original_argv for this region.
-        return argv[:1] + original_argv[-len(unparsed_args):]
+        ret_val = argv[:1] + original_argv[-len(unparsed_args):]
     else:
-      return argv[:1]
+      ret_val = argv[:1]
+
+    self._AssertAllValidators()
+    return ret_val
 
   def Reset(self):
     """Resets the values to the point before FLAGS(argv) was called."""
@@ -1109,6 +1264,8 @@ class FlagValues:
 
   def __RenderModuleFlags(self, module, flags, output_lines, prefix=""):
     """Generates a help string for a given module."""
+    if not isinstance(module, str):
+      module = module.__name__
     output_lines.append('\n%s%s:' % (prefix, module))
     self.__RenderFlagList(flags, output_lines, prefix + "  ")
 
@@ -1131,12 +1288,27 @@ class FlagValues:
     if key_flags:
       self.__RenderModuleFlags(module, key_flags, output_lines, prefix)
 
-  def MainModuleHelp(self):
-    """Returns: A string describing the key flags of the main module."""
+  def ModuleHelp(self, module):
+    """Describe the key flags of a module.
+
+    Args:
+      module: A module object or a module name (a string).
+
+    Returns:
+      string describing the key flags of a module.
+    """
     helplist = []
-    self.__RenderOurModuleKeyFlags(_GetMainModule(), helplist)
+    self.__RenderOurModuleKeyFlags(module, helplist)
     return '\n'.join(helplist)
 
+  def MainModuleHelp(self):
+    """Describe the key flags of the main module.
+
+    Returns:
+      string describing the key flags of a module.
+    """
+    return self.ModuleHelp(_GetMainModule())
+
   def __RenderFlagList(self, flaglist, output_lines, prefix="  "):
     fl = self.FlagDict()
     special_fl = _SPECIAL_FLAGS.FlagDict()
@@ -1309,11 +1481,15 @@ class FlagValues:
         flag_line_list.append(line.strip())
     return flag_line_list
 
-  def ReadFlagsFromFiles(self, argv):
+  def ReadFlagsFromFiles(self, argv, force_gnu=True):
     """Processes command line args, but also allow args to be read from file.
     Args:
-      argv: A list of strings, usually sys.argv, which may contain one
-        or more flagfile directives of the form --flagfile="./filename".
+      argv: A list of strings, usually sys.argv[1:], which may contain one or
+        more flagfile directives of the form --flagfile="./filename".
+        Note that the name of the program (sys.argv[0]) should be omitted.
+      force_gnu: If False, --flagfile parsing obeys normal flag semantics.
+        If True, --flagfile parsing instead follows gnu_getopt semantics.
+        *** WARNING *** force_gnu=False may become the future default!
 
     Returns:
 
@@ -1356,11 +1532,19 @@ class FlagValues:
         else:
           # This handles the case of (-)-flagfile=foo.
           flag_filename = self.ExtractFilename(current_arg)
-        new_argv = (new_argv[:1] +
-                    self.__GetFlagFileLines(flag_filename, parsed_file_list) +
-                    new_argv[1:])
+        new_argv[0:0] = self.__GetFlagFileLines(flag_filename, parsed_file_list)
       else:
         new_argv.append(current_arg)
+        # Stop parsing after '--', like getopt and gnu_getopt.
+        if current_arg == '--':
+          break
+        # Stop parsing after a non-flag, like getopt.
+        if not current_arg.startswith('-'):
+          if not force_gnu and not self.__dict__['__use_gnu_getopt']:
+            break
+
+    if rest_of_args:
+      new_argv.extend(rest_of_args)
 
     return new_argv
 
@@ -1436,6 +1620,19 @@ class FlagValues:
 
     outfile.write('</AllFlags>\n')
     outfile.flush()
+
+  def AddValidator(self, validator):
+    """Register new flags validator to be checked.
+
+    Args:
+      validator: gflags_validators.Validator
+    Raises:
+      AttributeError: if validators work with a non-existing flag.
+    """
+    for flag_name in validator.GetFlagsNames():
+      flag = self.FlagDict()[flag_name]
+      flag.validators.append(validator)
+
 # end of FlagValues definition
 
 
@@ -1522,6 +1719,7 @@ class Flag:
     self.serializer = serializer
     self.allow_override = allow_override
     self.value = None
+    self.validators = []
 
     self.SetDefault(default)
 
@@ -1541,7 +1739,7 @@ class Flag:
     try:
       self.value = self.parser.Parse(argument)
     except ValueError, e:  # recast ValueError as IllegalFlagValue
-      raise IllegalFlagValue("flag --%s: %s" % (self.name, e))
+      raise IllegalFlagValue("flag --%s=%s: %s" % (self.name, argument, e))
     self.present += 1
 
   def Unparse(self):
@@ -1571,8 +1769,14 @@ class Flag:
     # cowardly bail out until someone fixes the semantics of trying to
     # pass None to a C++ flag.  See swig_flags.Init() for details on
     # this behavior.
+    # TODO(olexiy): Users can directly call this method, bypassing all flags
+    # validators (we don't have FlagValues here, so we can not check
+    # validators).
+    # The simplest solution I see is to make this method private.
+    # Another approach would be to store reference to the corresponding
+    # FlagValues with each flag, but this seems to be an overkill.
     if value is None and self.allow_override:
-      raise DuplicateFlag(self.name)
+      raise DuplicateFlagCannotPropagateNoneToSwig(self.name)
 
     self.default = value
     self.Unparse()
@@ -1612,7 +1816,14 @@ class Flag:
                              inner_indent)
     if self.help:
       _WriteSimpleXMLElement(outfile, 'meaning', self.help, inner_indent)
-    _WriteSimpleXMLElement(outfile, 'default', self.default, inner_indent)
+    # The default flag value can either be represented as a string like on the
+    # command line, or as a Python object.  We serialize this value in the
+    # latter case in order to remain consistent.
+    if self.serializer and not isinstance(self.default, str):
+      default_serialized = self.serializer.Serialize(self.default)
+    else:
+      default_serialized = self.default
+    _WriteSimpleXMLElement(outfile, 'default', default_serialized, inner_indent)
     _WriteSimpleXMLElement(outfile, 'current', self.value, inner_indent)
     _WriteSimpleXMLElement(outfile, 'type', self.Type(), inner_indent)
     # Print extra flag features this flag may have.
@@ -1634,7 +1845,47 @@ class Flag:
 # End of Flag definition
 
 
-class ArgumentParser:
+class _ArgumentParserCache(type):
+  """Metaclass used to cache and share argument parsers among flags."""
+
+  _instances = {}
+
+  def __call__(mcs, *args, **kwargs):
+    """Returns an instance of the argument parser cls.
+
+    This method overrides behavior of the __new__ methods in
+    all subclasses of ArgumentParser (inclusive). If an instance
+    for mcs with the same set of arguments exists, this instance is
+    returned, otherwise a new instance is created.
+
+    If any keyword arguments are defined, or the values in args
+    are not hashable, this method always returns a new instance of
+    cls.
+
+    Args:
+      args: Positional initializer arguments.
+      kwargs: Initializer keyword arguments.
+
+    Returns:
+      An instance of cls, shared or new.
+    """
+    if kwargs:
+      return type.__call__(mcs, *args, **kwargs)
+    else:
+      instances = mcs._instances
+      key = (mcs,) + tuple(args)
+      try:
+        return instances[key]
+      except KeyError:
+        # No cache entry for key exists, create a new one.
+        return instances.setdefault(key, type.__call__(mcs, *args))
+      except TypeError:
+        # An object in args cannot be hashed, always return
+        # a new instance.
+        return type.__call__(mcs, *args)
+
+
+class ArgumentParser(object):
   """Base class used to parse and convert arguments.
 
   The Parse() method checks to make sure that the string argument is a
@@ -1644,7 +1895,13 @@ class ArgumentParser:
 
   Subclasses should also define a syntactic_help string which may be
   presented to the user to describe the form of the legal values.
+
+  Argument parser classes must be stateless, since instances are cached
+  and shared between flags. Initializer arguments are allowed, but all
+  member variables must be derived from initializer arguments only.
   """
+  __metaclass__ = _ArgumentParserCache
+
   syntactic_help = ""
 
   def Parse(self, argument):
@@ -1674,6 +1931,78 @@ class ListSerializer(ArgumentSerializer):
     return self.list_sep.join([str(x) for x in value])
 
 
+# Flags validators
+
+
+def RegisterValidator(flag_name,
+                      checker,
+                      message='Flag validation failed',
+                      flag_values=FLAGS):
+  """Adds a constraint, which will be enforced during program execution.
+
+  The constraint is validated when flags are initially parsed, and after each
+  change of the corresponding flag's value.
+  Args:
+    flag_name: string, name of the flag to be checked.
+    checker: method to validate the flag.
+      input  - value of the corresponding flag (string, boolean, etc.
+        This value will be passed to checker by the library). See file's
+        docstring for examples.
+      output - Boolean.
+        Must return True if validator constraint is satisfied.
+        If constraint is not satisfied, it should either return False or
+          raise gflags_validators.Error(desired_error_message).
+    message: error text to be shown to the user if checker returns False.
+      If checker raises gflags_validators.Error, message from the raised
+        Error will be shown.
+    flag_values: FlagValues
+  Raises:
+    AttributeError: if flag_name is not registered as a valid flag name.
+  """
+  flag_values.AddValidator(gflags_validators.SimpleValidator(flag_name,
+                                                             checker,
+                                                             message))
+
+
+def MarkFlagAsRequired(flag_name, flag_values=FLAGS):
+  """Ensure that flag is not None during program execution.
+
+  Registers a flag validator, which will follow usual validator
+  rules.
+  Args:
+    flag_name: string, name of the flag
+    flag_values: FlagValues
+  Raises:
+    AttributeError: if flag_name is not registered as a valid flag name.
+  """
+  RegisterValidator(flag_name,
+                    lambda value: value is not None,
+                    message='Flag --%s must be specified.' % flag_name,
+                    flag_values=flag_values)
+
+
+def _RegisterBoundsValidatorIfNeeded(parser, name, flag_values):
+  """Enforce lower and upper bounds for numeric flags.
+
+  Args:
+    parser: NumericParser (either FloatParser or IntegerParser). Provides lower
+      and upper bounds, and help text to display.
+    name: string, name of the flag
+    flag_values: FlagValues
+  """
+  if parser.lower_bound is not None or parser.upper_bound is not None:
+
+    def Checker(value):
+      if value is not None and parser.IsOutsideBounds(value):
+        message = '%s is not %s' % (value, parser.syntactic_help)
+        raise gflags_validators.Error(message)
+      return True
+
+    RegisterValidator(name,
+                      Checker,
+                      flag_values=flag_values)
+
+
 # The DEFINE functions are explained in mode details in the module doc string.
 
 
@@ -1725,7 +2054,8 @@ def DEFINE_flag(flag, flag_values=FLAGS):
     flag_values._RegisterFlagByModule(_GetCallingModule(), flag)
 
 
-def _InternalDeclareKeyFlags(flag_names, flag_values=FLAGS):
+def _InternalDeclareKeyFlags(flag_names,
+                             flag_values=FLAGS, key_flag_values=None):
   """Declares a flag as key for the calling module.
 
   Internal function.  User code should call DECLARE_key_flag or
@@ -1734,20 +2064,28 @@ def _InternalDeclareKeyFlags(flag_names, flag_values=FLAGS):
   Args:
     flag_names: A list of strings that are names of already-registered
       Flag objects.
-    flag_values: A FlagValues object.  This should almost never need
-      to be overridden.
+    flag_values: A FlagValues object that the flags listed in
+      flag_names have registered with (the value of the flag_values
+      argument from the DEFINE_* calls that defined those flags).
+      This should almost never need to be overridden.
+    key_flag_values: A FlagValues object that (among possibly many
+      other things) keeps track of the key flags for each module.
+      Default None means "same as flag_values".  This should almost
+      never need to be overridden.
 
   Raises:
     UnrecognizedFlagError: when we refer to a flag that was not
       defined yet.
   """
+  key_flag_values = key_flag_values or flag_values
+
   module = _GetCallingModule()
 
   for flag_name in flag_names:
     if flag_name not in flag_values:
       raise UnrecognizedFlagError(flag_name)
     flag = flag_values.FlagDict()[flag_name]
-    flag_values._RegisterKeyFlagForModule(module, flag)
+    key_flag_values._RegisterKeyFlagForModule(module, flag)
 
 
 def DECLARE_key_flag(flag_name, flag_values=FLAGS):
@@ -1770,6 +2108,15 @@ def DECLARE_key_flag(flag_name, flag_values=FLAGS):
     flag_values: A FlagValues object.  This should almost never
       need to be overridden.
   """
+  if flag_name in _SPECIAL_FLAGS:
+    # Take care of the special flags, e.g., --flagfile, --undefok.
+    # These flags are defined in _SPECIAL_FLAGS, and are treated
+    # specially during flag parsing, taking precedence over the
+    # user-defined flags.
+    _InternalDeclareKeyFlags([flag_name],
+                             flag_values=_SPECIAL_FLAGS,
+                             key_flag_values=flag_values)
+    return
   _InternalDeclareKeyFlags([flag_name], flag_values=flag_values)
 
 
@@ -1794,6 +2141,17 @@ def ADOPT_module_key_flags(module, flag_values=FLAGS):
   _InternalDeclareKeyFlags(
       [f.name for f in flag_values._GetKeyFlagsForModule(module.__name__)],
       flag_values=flag_values)
+  # If module is this flag module, take _SPECIAL_FLAGS into account.
+  if module == _GetThisModuleObjectAndName()[0]:
+    _InternalDeclareKeyFlags(
+        # As we associate flags with _GetCallingModule(), the special
+        # flags defined in this module are incorrectly registered with
+        # a different module.  So, we can't use _GetKeyFlagsForModule.
+        # Instead, we take all flags from _SPECIAL_FLAGS (a private
+        # FlagValues, where no other module should register flags).
+        [f.name for f in _SPECIAL_FLAGS.FlagDict().values()],
+        flag_values=_SPECIAL_FLAGS,
+        key_flag_values=flag_values)
 
 
 #
@@ -1931,12 +2289,44 @@ class HelpshortFlag(BooleanFlag):
         print flags
       sys.exit(1)
 
+#
+# Numeric parser - base class for Integer and Float parsers
+#
+
+
+class NumericParser(ArgumentParser):
+  """Parser of numeric values.
+
+  Parsed value may be bounded to a given upper and lower bound.
+  """
+
+  def IsOutsideBounds(self, val):
+    return ((self.lower_bound is not None and val < self.lower_bound) or
+            (self.upper_bound is not None and val > self.upper_bound))
+
+  def Parse(self, argument):
+    val = self.Convert(argument)
+    if self.IsOutsideBounds(val):
+      raise ValueError("%s is not %s" % (val, self.syntactic_help))
+    return val
+
+  def WriteCustomInfoInXMLFormat(self, outfile, indent):
+    if self.lower_bound is not None:
+      _WriteSimpleXMLElement(outfile, 'lower_bound', self.lower_bound, indent)
+    if self.upper_bound is not None:
+      _WriteSimpleXMLElement(outfile, 'upper_bound', self.upper_bound, indent)
+
+  def Convert(self, argument):
+    """Default implementation: always returns its argument unmodified."""
+    return argument
+
+# End of Numeric Parser
 
 #
 # FLOAT FLAGS
 #
 
-class FloatParser(ArgumentParser):
+class FloatParser(NumericParser):
   """Parser of floating point values.
 
   Parsed value may be bounded to a given upper and lower bound.
@@ -1946,20 +2336,19 @@ class FloatParser(ArgumentParser):
   syntactic_help = " ".join((number_article, number_name))
 
   def __init__(self, lower_bound=None, upper_bound=None):
+    super(FloatParser, self).__init__()
     self.lower_bound = lower_bound
     self.upper_bound = upper_bound
     sh = self.syntactic_help
-    if lower_bound != None and upper_bound != None:
+    if lower_bound is not None and upper_bound is not None:
       sh = ("%s in the range [%s, %s]" % (sh, lower_bound, upper_bound))
-    elif lower_bound == 1:
-      sh = "a positive %s" % self.number_name
-    elif upper_bound == -1:
-      sh = "a negative %s" % self.number_name
     elif lower_bound == 0:
       sh = "a non-negative %s" % self.number_name
-    elif upper_bound != None:
+    elif upper_bound == 0:
+      sh = "a non-positive %s" % self.number_name
+    elif upper_bound is not None:
       sh = "%s <= %s" % (self.number_name, upper_bound)
-    elif lower_bound != None:
+    elif lower_bound is not None:
       sh = "%s >= %s" % (self.number_name, lower_bound)
     self.syntactic_help = sh
 
@@ -1967,21 +2356,8 @@ class FloatParser(ArgumentParser):
     """Converts argument to a float; raises ValueError on errors."""
     return float(argument)
 
-  def Parse(self, argument):
-    val = self.Convert(argument)
-    if ((self.lower_bound != None and val < self.lower_bound) or
-        (self.upper_bound != None and val > self.upper_bound)):
-      raise ValueError("%s is not %s" % (val, self.syntactic_help))
-    return val
-
   def Type(self):
     return 'float'
-
-  def WriteCustomInfoInXMLFormat(self, outfile, indent):
-    if self.lower_bound is not None:
-      _WriteSimpleXMLElement(outfile, 'lower_bound', self.lower_bound, indent)
-    if self.upper_bound is not None:
-      _WriteSimpleXMLElement(outfile, 'upper_bound', self.upper_bound, indent)
 # End of FloatParser
 
 
@@ -1995,14 +2371,14 @@ def DEFINE_float(name, default, help, lower_bound=None, upper_bound=None,
   parser = FloatParser(lower_bound, upper_bound)
   serializer = ArgumentSerializer()
   DEFINE(parser, name, default, help, flag_values, serializer, **args)
-
+  _RegisterBoundsValidatorIfNeeded(parser, name, flag_values=flag_values)
 
 #
 # INTEGER FLAGS
 #
 
 
-class IntegerParser(FloatParser):
+class IntegerParser(NumericParser):
   """Parser of an integer value.
 
   Parsed value may be bounded to a given upper and lower bound.
@@ -2011,6 +2387,27 @@ class IntegerParser(FloatParser):
   number_name = "integer"
   syntactic_help = " ".join((number_article, number_name))
 
+  def __init__(self, lower_bound=None, upper_bound=None):
+    super(IntegerParser, self).__init__()
+    self.lower_bound = lower_bound
+    self.upper_bound = upper_bound
+    sh = self.syntactic_help
+    if lower_bound is not None and upper_bound is not None:
+      sh = ("%s in the range [%s, %s]" % (sh, lower_bound, upper_bound))
+    elif lower_bound == 1:
+      sh = "a positive %s" % self.number_name
+    elif upper_bound == -1:
+      sh = "a negative %s" % self.number_name
+    elif lower_bound == 0:
+      sh = "a non-negative %s" % self.number_name
+    elif upper_bound == 0:
+      sh = "a non-positive %s" % self.number_name
+    elif upper_bound is not None:
+      sh = "%s <= %s" % (self.number_name, upper_bound)
+    elif lower_bound is not None:
+      sh = "%s >= %s" % (self.number_name, lower_bound)
+    self.syntactic_help = sh
+
   def Convert(self, argument):
     __pychecker__ = 'no-returnvalues'
     if type(argument) == str:
@@ -2043,6 +2440,7 @@ def DEFINE_integer(name, default, help, lower_bound=None, upper_bound=None,
   parser = IntegerParser(lower_bound, upper_bound)
   serializer = ArgumentSerializer()
   DEFINE(parser, name, default, help, flag_values, serializer, **args)
+  _RegisterBoundsValidatorIfNeeded(parser, name, flag_values=flag_values)
 
 
 #
@@ -2057,6 +2455,7 @@ class EnumParser(ArgumentParser):
   """
 
   def __init__(self, enum_values=None):
+    super(EnumParser, self).__init__()
     self.enum_values = enum_values
 
   def Parse(self, argument):
@@ -2111,6 +2510,7 @@ class BaseListParser(ArgumentParser):
 
   def __init__(self, token=None, name=None):
     assert name
+    super(BaseListParser, self).__init__()
     self._token = token
     self._name = name
     self.syntactic_help = "a %s separated list" % self._name
@@ -2295,6 +2695,7 @@ DEFINE_flag(HelpshortFlag())
 DEFINE_flag(HelpXMLFlag())
 
 # Define special flags here so that help may be generated for them.
+# NOTE: Please do NOT use _SPECIAL_FLAGS from outside this module.
 _SPECIAL_FLAGS = FlagValues()
 
 
